#
# Ruby/Bsearch - a binary search library for Ruby.
#
# Copyright (C) 2001 Satoru Takabayashi <satoru@namazu.org>
#     All rights reserved.
#     This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of 
# the Ruby's licence.
#
# Example:
#
#   % irb --simple-prompt -r bsearch.rb
#   >> [1,2,3,3,3,4,5].bsearch_first {|x| x <=> 3}
#   => 2
#   >> [1,2,3,3,3,4,5].bsearch {|x| x <=> 3} # same as above
#   => 2
#   >> [1,2,3,3,3,4,5].bsearch_last  {|x| x <=> 3}
#   => 4
#   >> [1,2,3,3,3,4,5].bsearch {|x| x <=> 6}
#   => nil
#   >> [1,2,3,3,3,4,5].bsearch_range {|x| x <=> 3}
#   => 2..4

module Bsearch
  VERSION = '1.2'
end

class Array
  #
  # This method searches the FIRST occurrence which satisfies a
  # condition given by a block in binary fashion.
  # 
  # This algorithm is extracted from Jon Bentley's
  # Programming Pearls 2nd ed. p.93
  #
  def bsearch_first (range = Range.new(0, self.length() - 1))
    low  = range.first() -1
    high = range.last() + 1
    while low + 1 != high
      mid = (low + high) / 2
      if yield(self[mid]) < 0
	low = mid
      else 
	high = mid
      end
    end

    if high >= self.length || yield(self[high]) != 0
      return nil
    else 
      return high
    end
  end

  alias bsearch bsearch_first

  #
  # This method searches the LAST occurrence which satisfies a
  # condition given by a block in binary fashion.
  # 
  def bsearch_last (range = Range.new(0, self.length() - 1))
    low  = range.first() -1
    high = range.last() + 1
    while low + 1 != high
      mid = (low + high) / 2
      if yield(self[mid]) <= 0
	low = mid
      else 
	high = mid
      end
    end

    if low <= -1 || yield(self[low]) != 0
      return nil
    else
      return low
    end
  end

  #
  # Return the search result as a Range object.
  #
  def bsearch_range (range = Range.new(0, self.length() -1))
    first = bsearch_first(range)
    if first then
      return Range.new(first, bsearch_last(range))
    else
      return nil
    end
  end
end

